require_relative "InternalW3cValidator"

include Rake::DSL

def internalW3cValidation(*args, &block)	
	internal_w3c_validator_wrapper = InternalW3cValidatorWrapper.new(&block)
	task = Proc.new { internal_w3c_validator_wrapper.run }
	Rake::Task.define_task(*args, &task)
end		

class InternalW3cValidatorWrapper
	def initialize(&block)
		@block = block;
	end

	def run()			
		configuration = InternalW3cValidationConfiguration.new
		@block.call(configuration)
		errorReporter = ErrorReporterFactory.create(configuration)
		results = InternalW3cValidator.new.validate_pages(configuration.pages)
		results.each do | error |
			errorReporter.show(error)		
		end
		errorReporter.end
		throw 'W3C Validation failed' if results.length > 0 && configuration.failOnError
	end
end

class ConsoleErrorReporter
	def initialize
		@error_count = 0
	end

	def show(error)
		@error_count += 1
		puts error
	end

	def end
		puts "-----------------------------------------------"
		puts "There are #{@error_count} W3c Validation Errors"
		puts "-----------------------------------------------"
	end
end

class HtmlErrorReporter
	def initialize(output_file_name)
		@error_count = 0
		@output_file_name = output_file_name
		dirname = File.dirname(output_file_name)
		puts dirname
		FileUtils.mkdir_p(dirname)
		@report_html_file = ReportHtmlFile.new(output_file_name)
	end

	def show(error)
		@error_count += 1
		@report_html_file.add_error(error)
	end

	def end
		@report_html_file.save
		puts "--------------------------------------------------------------------------------------"
		puts "There are #{@error_count} W3c Validation Errors (see #{@output_file_name} for details)"
		puts "--------------------------------------------------------------------------------------"
	end
end

class ReportHtmlFile 
	def initialize(file_name)
		@error_count = 0
		@file = File.new(file_name, "w+")
		@file.puts "<HTML><BODY>"
		@file.puts "<H1>W3C Validation Results</H1>"
		@file.puts "<UL>"
	end

	def add_error(error)
		@error_count += 1
		@file.puts "<LI>#{error}</LI>"
	end

	def save
		@file.puts "</UL>"
		@file.puts "<H3>There are #{@error_count} w3c errors</H3>"
		@file.puts "</BODY></HTML>"
		@file.close()
	end
end

class ErrorReporterFactory
	def self.create(configuration)
		return ConsoleErrorReporter.new if configuration.outputFile.nil?
		HtmlErrorReporter.new(configuration.outputFile)
	end
end

class InternalW3cValidationConfiguration
	attr_accessor 	:pages,
					:failOnError,
					:outputFile
end