require 'peach'
require 'w3c_validators'

include W3CValidators

class InternalW3cValidator
	def initialize(pageContentsRepository = PageContentsRepository.new)
		@pageContentsRepository = pageContentsRepository
	end

	def validate_pages(pages)
		errors = []
		pages.peach do | page_uri |
			pageHtml = @pageContentsRepository.get(page_uri)
			result = MarkupValidator.new.validate_text(pageHtml)
			result.errors.each do | error |
				errors.push(ValidationError.new(
					page_uri, error.message, error.line
				))
			end
		end
		return errors
	end
end

class PageContentsRepository
	require 'open-uri'

	def get(pageUri)
		open(pageUri).read
	end
end

class ValidationError
	attr_reader :message, :line, :page

	def initialize(page, message, line)
		@page = page
		@message = message
		@line = line
	end

	def to_s 
		"#{page} : line #{line} : #{message}"
	end
end